/*
 * file CcClearPromptCallback.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * com.ibm.rational.wvcm.stp.cc.CcClearPromptCallback
 *
 *  Copyright IBM Corporation 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */

package com.ibm.rational.wvcm.stp.cc;

import java.util.List;

/**
 * <p>Interface for handling ClearPrompt user interaction requests.
 * </p>
 * <p>ClearCase trigger scripts may use ClearPrompt to interactively prompt the
 * user for various kinds of input or responses.  CM API applications that wish
 * to handle these ClearPrompt requests must implement this interface and
 * register the implementation object with the ClearCase provider via
 * <code>CcProvider.registerClearPromptCallback</code>.
 * </p>
 * <p>See the clearprompt manual page - <code>cleartool man clearprompt</code> -
 * for more information.
 * </p>
 */
public interface CcClearPromptCallback {

    /**
     * ClearPrompt message type.
     */
    public enum MsgType {
        BOX, OK, WARNING, ERROR
    }

    /**
     * Possible responses for {@link #promptToProceed} method.
     */
    public enum ProceedOrAbort {
        PROCEED, ABORT
    }

    /**
     * Possible responses for {@link #promptForYesOrNo} method.
     */
    public enum YesOrNo {
        YES, NO, ABORT
    }

    /**
     * Callback methods may throw this exception to indicate that the user
     * wishes to abort the current ClearCase operation - the operation that
     * caused the trigger to fire.
     */
    public class AbortException extends Exception {
    }

    /**
     * Prompt the user for one or more lines of text.
     * @param promptMsg Prompt message to display
     * @param defaultResponse Default response to display
     * @param multiLineOk Allow user to enter multiple lines
     * @param maskInput Mask user input for password entry
     * @return user's response string
     * @exception AbortException to indicate the user wants to abort the
     *            current ClearCase operation
     */
    public String 
    promptForText(
            String promptMsg,
            String defaultResponse,
            boolean multiLineOk,
            boolean maskInput)
    throws AbortException;

    /**
     * Ask the user for a "yes" or "no" response to a question.
     * @param promptMsg Prompt message to display
     * @param defaultChoice Default response to display
     * @param choices Optional subset of responses to display
     * @param msgType message type: OK, WARNING, ERROR, BOX
     * @return user's response: YES, NO, or ABORT
     */
    public YesOrNo 
    promptForYesOrNo(
            String promptMsg, 
            YesOrNo defaultChoice,
            List<YesOrNo> choices,
            MsgType msgType);

    /**
     * Ask the user whether to proceed with the current ClearCase operation
     * or abort it.
     * @param promptMsg Prompt message to display
     * @param defaultResponse Default response to display
     * @param choices Optional subset of responses to display
     * @param msgType message type: OK, WARNING, ERROR, BOX
     * @return user's response: PROCEED or ABORT
     */
    public ProceedOrAbort 
    promptForProceed(
            String promptMsg, 
            ProceedOrAbort defaultChoice,
            List<ProceedOrAbort> choices,
            MsgType msgType);

    /**
     * Ask the user to select a single item from a list of choices.
     * @param promptMsg Prompt message to display
     * @param choices Possible choices
     * @return user's selection from list of possible choices
     * @exception AbortException to indicate the user wants to abort the
     *            current ClearCase operation
     */
    public String
    promptForChoice(
            String promptMsg,
            List<String> choices)
    throws AbortException;

    /**
     * Ask the user to select zero or more items from a list of choices.
     * @param promptMsg Prompt message to display
     * @param choices Possible choices
     * @return user's selection(s) from list of possible choices
     * @exception AbortException to indicate the user wants to abort the
     *            current ClearCase operation
     */
    public List<String>
    promptForMultiChoice(
            String promptMsg,
            List<String> choices)
    throws AbortException;
}
