/*
 * file WvcmException.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import java.util.HashMap;

/**
 * An exception that indicates that a failure of some sort has occurred in the WVCM provider. 
 *  
 * A WvcmException contains information such as a code describing the reason for the failure
 * and optionally an array of nested exceptions to further explain the cause of the error.
 * 
 * @since 1.0
 */
public class WvcmException extends Exception {

    /** The reason code for a WvcmException. */
    public static enum ReasonCode
    {

        /**
         * The provider suffered an I/O failure, the operation may be retried.
         * <p>
         * This ReasonCode can be thrown by any method that reads information
         * from the server or a persistent store.
         */
        READ_FAILED("read-failed"), //$NON-NLS-1$
        //

        /**
         * The provider suffered an I/O failure, the operation may be retried.
         * <p>
         * This ReasonCode can be thrown by any method that writes information
         * to the server or a persistent store.
         */
        WRITE_FAILED("write-failed"), //$NON-NLS-1$

        /**
         * Even though the specification says the property is valid for the
         * targeted resource, the server does not support it.
         * <p>
         * This ReasonCode can be thrown by any method defined to return a
         * property value from a proxy, i.e., any method of the form getXyz().
         */
        PROPERTY_NOT_SUPPORTED_BY_SERVER("property-not-supported-by-server"), //$NON-NLS-1$

        /**
         * The property value is unavailable because it was not in the property
         * name list when the proxy was created.
         * <p>
         * This ReasonCode can be thrown by any method defined to return a
         * property value from a proxy, i.e., any method of the form getXyz().
         */
        PROPERTY_NOT_REQUESTED("property-not-requested"), //$NON-NLS-1$

        /**
         * The property is not valid for this resource.
         * <p>
         * This ReasonCode can be thrown by any method defined to return a
         * property value from a proxy, i.e., any method of the form getXyz().
         */
        PROPERTY_NOT_DEFINED_FOR_RESOURCE("property-not-defined-for-resource"), //$NON-NLS-1$

        /** The property value update would overwrite an earlier change. */
        PROPERTY_OVERWRITE_FORBIDDEN("property-overwrite-forbidden"), //$NON-NLS-1$

        /**
         * The user is not authorized to execute the attempted operation.
         * <p>
         * This ReasonCode can be thrown by any method.
         */
        UNAUTHORIZED("unauthorized"), //$NON-NLS-1$

        /**
         * The provider understood the request, but is refusing to fulfill it.
         * Authorization will not help and the request SHOULD NOT be repeated.
         * If the provider wishes to make public why the operation has not been
         * successful, it SHOULD describe the reason for the refusal in the
         * message. If the provider does not wish to make this information
         * available to the client, the reason code {@link #NOT_FOUND} can be
         * used instead.
         * <p>
         * This ReasonCode can be thrown by any method that contacts the server,
         * i.e. any doXyz() method.
         */
        FORBIDDEN("forbidden"), //$NON-NLS-1$

        /**
         * The corresponding remote resource no longer exists or was never
         * created.
         */
        /**
         * The provider has not found anything matching the location of the
         * proxy. No indication is given whether the condition is temporary or
         * permanent. This reason code is commonly used when the server does not
         * wish to reveal exactly why the operation has been refused, or when no
         * other response is applicable.
         * <p>
         * This ReasonCode can be thrown by any method that contacts the server,
         * i.e. any doXyz() method.
         */
        NOT_FOUND("not-found"), //$NON-NLS-1$

        /**
         * The operation could not be completed because of a conflict with the
         * current state of the resource. This code is only allowed in
         * situations where it is expected that the user might be able to
         * resolve the conflict and attempt the operation again. The exception
         * message and data SHOULD include enough information for the user to
         * recognize the source of the conflict. Ideally, the exception would
         * include enough information for the user or user agent to fix the
         * problem; however, that might not be possible and is not required.
         * Conflicts are most likely to occur in response to an update
         * operation. For example, if versioning were being used and the entity
         * being updated included changes to a resource that conflict with those
         * made by an earlier (third-party) operation, the provider might use
         * the CONFLICT reason code to indicate that it can't complete the
         * request. In this case, the exception would likely contain a list of
         * the differences between the two versions in a format defined by the
         * exception subclass documentation.
         * <p>
         * This ReasonCode can be thrown by any method that contacts the server,
         * i.e. any doXyz() method.
         */
        CONFLICT("conflict"), //$NON-NLS-1$

        /**
         * The type of the persistent resource identified by this argument was
         * not compatible with the specified argument type.
         */
        BAD_ARGUMENT_TYPE("bad-argument-type"), //$NON-NLS-1$

        /** This folder already has a configuration. */
        CONTROLLED_CONFIGURATION_ALREADY_EXISTS(
                        "controlled-configuration-already-exists"), //$NON-NLS-1$

        /**
         * A baseline controlled folder already exists in this workspace for
         * this baseline history.
         */
        CANNOT_HAVE_MULTIPLE_BASELINE_CONTROLLED_FOLDERS(
                        "cannot-have-multiple-baseline-controlled-folders"), //$NON-NLS-1$

        /** Cannot create this resource at the specified location. */
        CANNOT_CREATE_AT_THIS_LOCATION("cannot-create-at-this-location"), //$NON-NLS-1$

        /**
         * Failed to perform the merge because the target could not be
         * checked-out.
         */
        CANNOT_MERGE_CHECKOUT_NOT_ALLOWED("checkout-not-allowed"), //$NON-NLS-1$

        /**
         * Failed to checkout resource because multiple checkout is discouraged
         * and the caller did not specify fork-ok.
         */
        CANNOT_CHECKOUT_MULTI_CHECKOUT_IS_DISCOURAGED(
                        "checkout-of-checked-out-version-is-discouraged"), //$NON-NLS-1$

        /**
         * A potentially recoverable condition prevented the server from
         * retrieving the property. This ReasonCode can be thrown by any method
         * defined to return a property value from a proxy, i.e., any method of
         * the form getXyz().
         */
        PROPERTY_NOT_CURRENTLY_AVAILABLE("property-not-currently-available"), //$NON-NLS-1$

        /** Checkout of an already checked-out resource is forbidden. */
        CANNOT_CHECKOUT_MULTI_CHECKOUT_IS_FORBIDDEN(
                        "cannot-checkout-multi-checkout-is-forbidden"), //$NON-NLS-1$

        /**
         * Cannot remove the specified label because it does not used by this
         * resource.
         */
        CANNOT_REMOVE_LABEL_DOES_NOT_EXIST("cannot-remove-label-does-not-exist"), //$NON-NLS-1$

        /**
         * Cannot checkin since it would cause a fork and forking is
         * discouraged.
         */
        CANNOT_CHECKIN_FORK_DISCOURAGED("checkin-fork-discouraged"), //$NON-NLS-1$

        /** A fork in the version tree is not allowed. */
        CANNOT_FORK("checkin-fork-forbidden"), //$NON-NLS-1$

        /**
         * Failed to modify content/properties because the resource specified
         * was a version.
         */
        CANNOT_MODIFY_VERSION("cannot-modify-version"), //$NON-NLS-1$

        /**
         * Method failed on some of the specified resources.
         */
        MULTI_STATUS("multi-status"), //$NON-NLS-1$

        /**
         * Creating a resource failed because a resource already exists at the
         * specified location.
         */
        RESOURCE_ALREADY_EXISTS_AT_LOCATION(
                        "resource-already-exists-at-location"), //$NON-NLS-1$

        /**
         * Failed to checkout because descendant already exists and forking is
         * discouraged.
         */
        CANNOT_CHECKOUT_FORKING_IS_DISCOURAGED(
                        "checkout-of-version-with-descendant-is-discouraged"), //$NON-NLS-1$

        /**
         * Failed to checkout because descendant already exists and forking is
         * forbidden.
         */
        CANNOT_CHECKOUT_FORKING_IS_FORBIDDEN(
                        "cannot-checkout-forking-is-forbidden"), //$NON-NLS-1$

        /**
         * Cannot checkin because the resources predecessors are not descendants
         * of the root of the version history.
         */
        VERSION_HISTORY_MUST_BE_A_TREE("version-history-must-be-a-tree"), //$NON-NLS-1$

        /**
         * baseline control failed because the folder already has controlled
         * resources.
         */
        CANNOT_HAVE_CONTROLLED_MEMBERS("cannot-have-controlled-members"), //$NON-NLS-1$

        /**
         * The operation failed because it would result in more than one
         * controlled resource for this version history in a workspace.
         */
        ONE_CONTROLLED_RESOURCE_PER_HISTORY_PER_WORKSPACE(
                        "one-controlled-resource-per-history-per-workspace"), //$NON-NLS-1$

        /**
         * Failed because more than one version of this resource is referenced
         * in the specified activity.
         */
        ONE_CHECKOUT_PER_ACTIVITY_PER_HISTORY(
                        "one-checkout-per-activity-per-history"), //$NON-NLS-1$

        /**
         * Failed because there is a reserved checkout of a version in this
         * version history.
         */
        CANNOT_CHECKIN_TO_RESERVED_ACTIVITY(
                        "cannot-checkin-to-reserved-activity"), //$NON-NLS-1$

        /**
         * The operation failed because the resource must be in the checked-in
         * state.
         */
        MUST_BE_CHECKED_IN("must-be-checked-in"), //$NON-NLS-1$

        /**
         * The operation failed because the resource must be in the checked-out
         * state.
         */
        MUST_BE_CHECKED_OUT("must-be-checked-out"), //$NON-NLS-1$

        /**
         * Each version in an activity for a given version history must be on
         * the same line of descent.
         */
        CANNOT_CREATE_BRANCH_IN_ACTIVITY("linear-activity"), //$NON-NLS-1$

        /**
         * Each version in a stream for a given version history must be on the
         * same line of descent.
         */
        CANNOT_CREATE_BRANCH_IN_STREAM("linear-stream"), //$NON-NLS-1$

        /** The label is already in use by this resource. */
        ADD_MUST_BE_NEW_LABEL("add-must-be-new-label"), //$NON-NLS-1$

        /**
         * Failed to checkin the configuration because some of it's members are
         * still checked-out.
         */
        NO_CHECKED_OUT_BASELINE_CONTROLLED_FOLDER_MEMBERS(
                        "no-checked-out-baseline-controlled-folder-members"), //$NON-NLS-1$

        /** Operation failed because it attempted to set a protected property. */
        CANNOT_MODIFY_PROTECTED_PROPERTY("cannot-modify-protected-property"), //$NON-NLS-1$

        /**
         * Report failed since the resource does not support the specified
         * report.
         */
        METHOD_NOT_SUPPORTED("method-not-supported"), //$NON-NLS-1$

        /** Illegal syntax for location string value. */
        ILLEGAL_LOCATION_SYNTAX("illegal-location-syntax"), //$NON-NLS-1$

        /** Cannot create cross-server binding. */
        NO_CROSS_SERVER_BINDING("no-cross-server-binding"), //$NON-NLS-1$

        /** Cannot overwrite existing binding. */
        CANNOT_OVERWRITE("cannot-overwrite"), //$NON-NLS-1$

        /** Cannot create location cycle. */
        CYCLE_NOT_ALLOWED("cycle-not-allowed"), //$NON-NLS-1$

        /** Method execution was aborted via notification to the Feedback object. */
        ABORTED("aborted"); //$NON-NLS-1$


        /**
         * Map of RFC 3253 pre & post condition names to
         * WvcmException.ReasonCode.
         */
        private static final HashMap<String, ReasonCode> _rfc3253ToWvcmMap 
            = new HashMap<String, ReasonCode>();

        static
        {
            //  VERSION-CONTROL
            _rfc3253ToWvcmMap.put("put-under-version-control", FORBIDDEN ); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("must-not-change-existing-checked-in-out", FORBIDDEN); //$NON-NLS-1$
            // REPORT
            _rfc3253ToWvcmMap.put("supported-report", METHOD_NOT_SUPPORTED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("acceptable-depth", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("no-modification", FORBIDDEN); //$NON-NLS-1$
            // PUT
            _rfc3253ToWvcmMap.put("cannot-modify-version-controlled-content", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cannot-modify-version", CANNOT_MODIFY_VERSION); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("auto-checkout", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("auto-checkout-checkin", FORBIDDEN); //$NON-NLS-1$
            // PROPFIND
            _rfc3253ToWvcmMap.put("supported-live-property", PROPERTY_NOT_CURRENTLY_AVAILABLE); //$NON-NLS-1$
            // PROPPATCH
            _rfc3253ToWvcmMap.put("cannot-modify-version-controlled-property", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cannot-modify-version", CANNOT_MODIFY_VERSION); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cannot-modify-protected-property", CANNOT_MODIFY_PROTECTED_PROPERTY); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("supported-live-property", PROPERTY_NOT_CURRENTLY_AVAILABLE); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("auto-checkout", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("auto-checkout-checkin", FORBIDDEN); //$NON-NLS-1$
            // DELETE
            _rfc3253ToWvcmMap.put("no-version-delete", METHOD_NOT_SUPPORTED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("update-predecessor-set", FORBIDDEN); //$NON-NLS-1$
            // COPY
            _rfc3253ToWvcmMap.put("must-not-copy-versioning-property", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("auto-checkout", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("auto-checkout-checkin", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("copy-creates-new-resource", FORBIDDEN); //$NON-NLS-1$
            // MOVE
            _rfc3253ToWvcmMap.put("cannot-rename-version", METHOD_NOT_SUPPORTED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("preserve-versioning-properties", FORBIDDEN); //$NON-NLS-1$
            // UNLOCK
            _rfc3253ToWvcmMap.put("version-history-is-tree", VERSION_HISTORY_MUST_BE_A_TREE); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("auto-checkin", FORBIDDEN); //$NON-NLS-1$
            // CHECKOUT
            _rfc3253ToWvcmMap.put("must-be-checked-in", MUST_BE_CHECKED_IN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkout-of-version-with-descendant-is-forbidden", CANNOT_CHECKOUT_FORKING_IS_FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkout-of-version-with-descendant-is-discouraged", CANNOT_CHECKOUT_FORKING_IS_DISCOURAGED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkout-of-checked-out-version-is-forbidden", CANNOT_CHECKOUT_MULTI_CHECKOUT_IS_FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkout-of-checked-out-version-is-discouraged", CANNOT_CHECKOUT_MULTI_CHECKOUT_IS_DISCOURAGED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("is-checked-out", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("initialize-predecessor-set", FORBIDDEN); //$NON-NLS-1$
            // CHECKIN
            _rfc3253ToWvcmMap.put("must-be-checked-out", MUST_BE_CHECKED_OUT); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("version-history-is-tree", VERSION_HISTORY_MUST_BE_A_TREE); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkin-fork-forbidden", CANNOT_FORK); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkin-fork-discouraged", CANNOT_CHECKIN_FORK_DISCOURAGED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("create-version", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("initialize-version-content-and-properties", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checked-in", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("keep-checked-out", FORBIDDEN); //$NON-NLS-1$
            // UNCHECKOUT
            _rfc3253ToWvcmMap.put("must-be-checked-out-version-controlled-resource", MUST_BE_CHECKED_OUT); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cancel-checked-out", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("restore-content-and-dead-properties", FORBIDDEN); //$NON-NLS-1$
            // REPORT (DAV:locate-by-history)
            _rfc3253ToWvcmMap.put("must-be-version-history", BAD_ARGUMENT_TYPE); //$NON-NLS-1$
            // DELETE (version history feature)
            _rfc3253ToWvcmMap.put("delete-version-set", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("version-history-has-root", FORBIDDEN); //$NON-NLS-1$
            // COPY (version history feature)
            _rfc3253ToWvcmMap.put("cannot-copy-history", METHOD_NOT_SUPPORTED); //$NON-NLS-1$
            // MOVE (version history feature)
            _rfc3253ToWvcmMap.put("cannot-rename-history", METHOD_NOT_SUPPORTED); //$NON-NLS-1$
            // VERSION-CONTROL (version history feature)
            _rfc3253ToWvcmMap.put("new-version-history", FORBIDDEN); //$NON-NLS-1$
            // CHECKIN (version history feature)
            _rfc3253ToWvcmMap.put("add-to-history", FORBIDDEN); //$NON-NLS-1$
            // MKWORKSPACE
            _rfc3253ToWvcmMap.put("resource-must-be-null", RESOURCE_ALREADY_EXISTS_AT_LOCATION); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("workspace-location-ok", CANNOT_CREATE_AT_THIS_LOCATION); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("initialize-workspace", FORBIDDEN); //$NON-NLS-1$
            // DELETE (workspace)
            _rfc3253ToWvcmMap.put("delete-workspace-members", FORBIDDEN); //$NON-NLS-1$
            // MOVE (workspace)
            _rfc3253ToWvcmMap.put("workspace-member-moved", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("workspace-moved", FORBIDDEN); //$NON-NLS-1$
            // VERSION-CONTROL (workspace)
            _rfc3253ToWvcmMap.put("cannot-add-to-existing-history", RESOURCE_ALREADY_EXISTS_AT_LOCATION); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("must-be-version", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("one-version-controlled-resource-per-history-per-workspace", ONE_CONTROLLED_RESOURCE_PER_HISTORY_PER_WORKSPACE); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("new-version-controlled-resource", FORBIDDEN); //$NON-NLS-1$
            // UPDATE (workspace)
            _rfc3253ToWvcmMap.put("update-content-and-properties", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("report-properties", FORBIDDEN); //$NON-NLS-1$
            // LABEL
            _rfc3253ToWvcmMap.put("must-be-checked-in", MUST_BE_CHECKED_IN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("add-must-be-new-label", ADD_MUST_BE_NEW_LABEL); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("label-must-exist", CANNOT_REMOVE_LABEL_DOES_NOT_EXIST); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("add-or-set-label", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("remove-label", FORBIDDEN); //$NON-NLS-1$
            // GET (label)
            _rfc3253ToWvcmMap.put("apply-request-to-labeled-version", FORBIDDEN); //$NON-NLS-1$
            // PROPFIND (label)
            _rfc3253ToWvcmMap.put("must-select-version-in-history", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("apply-request-to-labeled-version", FORBIDDEN); //$NON-NLS-1$
            // COPY (label)
            _rfc3253ToWvcmMap.put("apply-request-to-labeled-version", FORBIDDEN); //$NON-NLS-1$
            // CHECKOUT (label)
            _rfc3253ToWvcmMap.put("must-not-have-label-and-apply-to-version", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("apply-request-to-labeled-version", FORBIDDEN); //$NON-NLS-1$
            // UPDATE (label)
            _rfc3253ToWvcmMap.put("depth-update", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("apply-request-to-labeled-version", FORBIDDEN); //$NON-NLS-1$
            // CHECKOUT (working resource)
            _rfc3253ToWvcmMap.put("checkout-of-version-with-descendant-is-discouraged", CANNOT_CHECKOUT_FORKING_IS_DISCOURAGED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkout-of-checked-out-version-is-forbidden", CANNOT_CHECKOUT_MULTI_CHECKOUT_IS_FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkout-of-checked-out-version-is-discouraged", CANNOT_CHECKOUT_MULTI_CHECKOUT_IS_DISCOURAGED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("create-working-resource", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("create-working-resource-from-checked-in-version", FORBIDDEN); //$NON-NLS-1$
            // CHECKIN (working resource)
            _rfc3253ToWvcmMap.put("must-be-checked-in", MUST_BE_CHECKED_IN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("version-history-is-tree", VERSION_HISTORY_MUST_BE_A_TREE); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkin-fork-forbidden", CANNOT_FORK); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkin-fork-discouraged", CANNOT_CHECKIN_FORK_DISCOURAGED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("no-overwrite-by-auto-update", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("create-version", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("initialize-version-content-and-properties", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("auto-update", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("delete-working-resource", FORBIDDEN); //$NON-NLS-1$
            // COPY (working resource)
            _rfc3253ToWvcmMap.put("copy-creates-new-resource", FORBIDDEN); //$NON-NLS-1$
            // MOVE (working resource)
            _rfc3253ToWvcmMap.put("cannot-rename-working-resource", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("update-auto-update", FORBIDDEN); //$NON-NLS-1$
            // MERGE
            _rfc3253ToWvcmMap.put("cannot-merge-checked-out-resource", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checkout-not-allowed", CANNOT_MERGE_CHECKOUT_NOT_ALLOWED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("ancestor-version", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("decendent-version", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("checked-out-for-merge", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("update-merge-set", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("report-properties", FORBIDDEN); //$NON-NLS-1$
            // DELETE (merge)
            _rfc3253ToWvcmMap.put("delete-version-reference", FORBIDDEN); //$NON-NLS-1$
            // CHECKIN (merge)
            _rfc3253ToWvcmMap.put("merge-must-be-complete", FORBIDDEN); //$NON-NLS-1$
            // BASELINE-CONTROL
            _rfc3253ToWvcmMap.put("configuration-must-not-exist", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("must-be-baseline", BAD_ARGUMENT_TYPE); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("must-have-no-version-controlled-members", CANNOT_HAVE_CONTROLLED_MEMBERS); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("one-baseline-controlled-collection-per-history-per-workspace", CANNOT_HAVE_MULTIPLE_BASELINE_CONTROLLED_FOLDERS); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("create-configuration", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("reference-configuration", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("select-existing-baseline", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("create-new-baseline", FORBIDDEN); //$NON-NLS-1$
            // REPORT (DAV:compare-baseline report)
            _rfc3253ToWvcmMap.put("must-be-baseline", BAD_ARGUMENT_TYPE); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("baselines-from-same-history", FORBIDDEN); //$NON-NLS-1$
            // CHECKOUT (baseline)
            _rfc3253ToWvcmMap.put("must-not-update-baseline-collection", FORBIDDEN); //$NON-NLS-1$
            // CHECKIN (baseline)
            _rfc3253ToWvcmMap.put("no-checked-out-baseline-controlled-collection-members", NO_CHECKED_OUT_BASELINE_CONTROLLED_FOLDER_MEMBERS); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("one-version-per-history-per-baseline", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cannot-modify-configuration", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("create-baseline-collection", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("modify-configuration", FORBIDDEN); //$NON-NLS-1$
            // UPDATE (baseline)
            _rfc3253ToWvcmMap.put("baseline-controlled-members-must-be-checked-in", NO_CHECKED_OUT_BASELINE_CONTROLLED_FOLDER_MEMBERS); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("must-not-update-baseline-collection", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cannot-modify-version-controlled-configuration", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("set-baseline-controlled-collection-members", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("modify-configuration", FORBIDDEN); //$NON-NLS-1$
            // MERGE (baseline)
            _rfc3253ToWvcmMap.put("must-not-update-baseline-collection", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cannot-modify-configuration", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("merge-baseline", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("set-baseline-controlled-collection-members", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("modify-configuration", FORBIDDEN); //$NON-NLS-1$
            // ACTIVITY
            _rfc3253ToWvcmMap.put("resource-must-be-null", RESOURCE_ALREADY_EXISTS_AT_LOCATION); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("activity-location-ok", CANNOT_CREATE_AT_THIS_LOCATION); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("initialize-activity", FORBIDDEN); //$NON-NLS-1$
            // REPORT (DAV:latest-activity-version)
            _rfc3253ToWvcmMap.put("must-be-activity", BAD_ARGUMENT_TYPE); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("delete-activity-reference", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("update-checked-out-reference", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("update-activity-reference", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("update-workspace-reference", FORBIDDEN); //$NON-NLS-1$
            // CHECKOUT (activity)
            _rfc3253ToWvcmMap.put("one-checkout-per-activity-per-history", ONE_CHECKOUT_PER_ACTIVITY_PER_HISTORY); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("linear-activity", CANNOT_CREATE_BRANCH_IN_ACTIVITY); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("initialize-activity-set", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("initialize-reserved", FORBIDDEN); //$NON-NLS-1$
            // CHECKIN (activity)
            _rfc3253ToWvcmMap.put("linear-activity", CANNOT_CREATE_BRANCH_IN_ACTIVITY); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("atomic-activity-checkin", MULTI_STATUS); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("initialize-activity-set", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("activity-checkin", FORBIDDEN); //$NON-NLS-1$
            // MERGE (activity)
            _rfc3253ToWvcmMap.put("checkin-activity", FORBIDDEN); //$NON-NLS-1$
            // DELETE (version-controlled-collection)
            _rfc3253ToWvcmMap.put("cannot-modify-checked-in-parent", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("delete-working-collection-binding", FORBIDDEN); //$NON-NLS-1$
            // COPY (version-controlled-collection)
            _rfc3253ToWvcmMap.put("cannot-copy-collection-version", METHOD_NOT_SUPPORTED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cannot-modify-checked-in-parent", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cannot-modify-destination-checked-in-parent", FORBIDDEN); //$NON-NLS-1$
            // VERSION-CONTROL (version-controlled-collection)
            _rfc3253ToWvcmMap.put("cannot-modify-checked-in-parent", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("new-version-controlled-collection", FORBIDDEN); //$NON-NLS-1$
            // CHECKOUT (version-controlled-collection)
            _rfc3253ToWvcmMap.put("initialize-version-history-bindings", FORBIDDEN); //$NON-NLS-1$
            // CHECKIN (version-controlled-collection)
            _rfc3253ToWvcmMap.put("initialize-version-controlled-bindings", FORBIDDEN); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("version-control-working-collection-members", FORBIDDEN); //$NON-NLS-1$
            // UNCHECKOUT, UPDATE, and MERGE (version-controlled-collection)
            _rfc3253ToWvcmMap.put("update-version-controlled-collection-members", FORBIDDEN); //$NON-NLS-1$
            // BIND
            _rfc3253ToWvcmMap.put("bind-source-exists", NOT_FOUND); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("name-allowed", CANNOT_CREATE_AT_THIS_LOCATION); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("binding-allowed", METHOD_NOT_SUPPORTED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cross-server-binding", NO_CROSS_SERVER_BINDING); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("can-overwrite", CANNOT_OVERWRITE); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("cycle-allowed", CYCLE_NOT_ALLOWED); //$NON-NLS-1$
            _rfc3253ToWvcmMap.put("new-binding", FORBIDDEN); //$NON-NLS-1$
            // UNBIND
            _rfc3253ToWvcmMap.put("unbind-source-exists", NOT_FOUND); //$NON-NLS-1$
            // REBIND
            _rfc3253ToWvcmMap.put("rebind-source-exists", NOT_FOUND); //$NON-NLS-1$
        }

        /**  Hidden constuctor for type-safety. */
        private ReasonCode(String codeImage) {
            _codeImage = codeImage;
        }

        /**
         * The ReasonCode for an RFC 3253 pre- or post-condition.
         * 
         * @param condition the RFC 3253 pre- or post- condition local name.
         * @return the corresponding ReasonCode.
         */
        public static ReasonCode getReasonCodeForRFC3253Condition(String condition)
        {
            return _rfc3253ToWvcmMap.get(condition);
        }

        /**
         * Returns a string representation of this ReasonCode suitable for diagnostics.
         */
        @Override
        public String toString() {
            return _codeImage;
        }

        /**
         * Return the string form of this ReasonCode.
         */
        private final String _codeImage;
    }

    /**
     * The resource that caused the exception.
     */
    private Resource _resource;

    /** 
     * The reason for the exception.
     */
    private ReasonCode _reasonCode;

    /** 
     * An array of nested exceptions that further explain the cause of the error.
     */
    private Throwable[] _nestedExceptions = null;

    /**
     * Constructs a WvcmException.
     * 
     * @param message the detail message for the exception.
     * @param reasonCode the reason for the exception.
     */
    public WvcmException(
            String message,
            ReasonCode reasonCode) {
        super(message);
        _resource = null;
        _reasonCode = reasonCode;
        _nestedExceptions = null;
    }

    /**
     * Constructs a WvcmException.
     * 
     * @param message the detail message for the exception.
     * @param resource the resource causing the exception.
     * @param reasonCode the reason for the exception.
     */
    public WvcmException(
            String message,
            Resource resource,
            ReasonCode reasonCode) {
        super(message);
        _resource = resource;
        _reasonCode = reasonCode;
        _nestedExceptions = null;
    }

    /**
     * Constructs a WvcmException.
     * 
     * @param message the detail message for the exception.
     * @param resource the resource causing the exception.
     * @param reasonCode the reason for the exception.
     * @param cause the exception that caused this exception.
     */
    public WvcmException(
            String message,
            Resource resource,
            ReasonCode  reasonCode,
            Throwable cause) {
        super(message, cause);
        _resource = resource;
        _reasonCode = reasonCode;
        _nestedExceptions = null;
    }

    /**
     * Constructs a WvcmException.
     * 
     * @param message the detail message for the exception.
     * @param resource the resource causing the exception.
     * @param reasonCode the reason for the exception.
     * @param nestedExceptions any nested exception.
     */
    public WvcmException(
            String message,
            Resource resource,
            ReasonCode  reasonCode,
            Throwable[] nestedExceptions) {
        super(message);
        _resource = resource;
        _reasonCode = reasonCode;
        _nestedExceptions = nestedExceptions;
    }

    /**
     * Constructs a WvcmException.
     * 
     * @param message the detail message for the exception.
     * @param resource the resource causing the exception.
     * @param reasonCode the reason for the exception.
     * @param cause the exception that caused this exception.
     * @param nestedExceptions any nested exception.
     */
    public WvcmException(
            String message,
            Resource resource,
            ReasonCode  reasonCode,
            Throwable cause,
            Throwable[] nestedExceptions) {
        super(message, cause);
        _resource = resource;
        _reasonCode = reasonCode;
        _nestedExceptions = nestedExceptions;
    }

    /**
     * Get the resource causing the exception.
     * 
     * @return the resource causing the exception.
     */
    public Resource getResource() {
        return _resource;
    }

    /**
     * Get the reason code that describes the nature of the error.
     * 
     * @return the reason code that describes the nature of the error.
     */
    public ReasonCode getReasonCode() {
        return _reasonCode;
    }

    /**
     * Get any nested exceptions that further explain the cause of the error.
     * 
     * @return any nested exceptions that further explain the cause of the error.
     */
    public Throwable[] getNestedExceptions() {
        return _nestedExceptions;
    }

    /**
     * The serial version UID.
     */
    private static final long serialVersionUID = -663023054869664237L;
}
